/* hgKnownGene - Load auxiliarry tables on Genie known genes. */
#include "common.h"
#include "obscure.h"
#include "dystring.h"
#include "linefile.h"
#include "hash.h"
#include "hgap.h"
#include "jksql.h"
#include "gff.h"

void usage()
/* Explain usage and exit. */
{
errAbort(
  "hgKnownGene - Load auxiliarry tables on Genie known genes\n"
  "usage:\n"
  "   hgKnownGene database known.gtf\n");
}

struct hash *loadNameValTable(struct sqlConnection *conn, char *tableName, int hashSize)
/* Create a hash and load it up from table. */
{
char query[128];
struct sqlResult *sr;
char **row;
struct hash *hash = newHash(hashSize);

sprintf(query, "select id,name from %s", tableName);
sr = sqlGetResult(conn, query);
while ((row = sqlNextRow(sr)) != NULL)
    {
    hashAdd(hash, row[1], intToPt(sqlUnsigned(row[0])));
    }
sqlFreeResult(&sr);
return hash;
}

static void readQuotedString(struct lineFile *lf, char *in, char *out, char **retNext)
/* Parse quoted string and abort on error. */
{
if (!parseQuotedString(in, out, retNext))
    errAbort("Line %d of %s\n", lf->lineIx, lf->fileName);
}

void hgKnownGeneInDb(char *fileName)
/* hgKnownGene - Load auxiliarry tables on Genie known genes. */
{
struct sqlConnection *conn = hgStartUpdate();
struct hash *productHash = loadNameValTable(conn, "productName", 16);
struct hash *geneHash = loadNameValTable(conn, "geneName", 16);
FILE *productTab = hgCreateTabFile("productName");
FILE *geneTab = hgCreateTabFile("geneName");
FILE *knownInfoTab = hgCreateTabFile("knownInfo");
struct lineFile *lf = lineFileOpen(fileName, TRUE);
char *firstWords[16], *line;
int wordCount, lineSize;
char *type, *val;
struct hashEl *hel;

while (lineFileNext(lf, &line, &lineSize))
    {
    if (line[0] == '#')
        continue;
    wordCount = chopTabs(line, firstWords);
    if (wordCount < 9)
        continue;
    if (sameWord("prim_trans", firstWords[2]))
        {
	char *s = firstWords[8];
	char *transId = NULL, *geneId = "n/a", *productName = "n/a", 
	    *geneName = "n/a", *pgi = "n/a", *ngi = "n/a", 
	    *proteinId = "n/a";
	   
	for (;;)
	   {
	   if ((type = nextWord(&s)) == NULL)
	       break;
	   s = skipLeadingSpaces(s);
	   if (s[0] == 0)
	       errAbort("Unpaired type/val on end of gtf line %d of %s", lf->lineIx, lf->fileName);
	   if (s[0] == '"' || s[0] == '\'')
	       {
	       val = s;
	       readQuotedString(lf, s, val, &s);
	       subChar(val, '\t', ' ');
	       }
	   else
	       val = nextWord(&s);
	   if (s != NULL)
	      {
	      s = strchr(s, ';');
	      if (s != NULL)
		 ++s;
	      }
	   if (sameString("gene_id", type))
	       geneId = val;
	   else if (sameString("pgi", type))
	       pgi = val;
	   else if (sameString("gene_name", type))
	       geneName = val;
	   else if (sameString("product", type))
	       productName = val;
	   else if (sameString("transcript_id", type))
	       transId = val;
	   else if (sameString("ngi", type))
	       ngi = val;
	   else if (sameString("protein_id", type))
	       proteinId = val;
	   }
	if (transId != NULL)
	    {
	    HGID  productNameId, geneNameId;
	    if ((hel = hashLookup(geneHash, geneName)) != NULL)
	        geneNameId = ptToInt(hel->val);
	    else
	        {
		geneNameId = hgNextId();
		fprintf(geneTab, "%u\t%s\n", geneNameId, geneName);
		hashAdd(geneHash, geneName, intToPt(geneNameId));
		}
	    if ((hel = hashLookup(productHash, productName)) != NULL)
	        productNameId = ptToInt(hel->val);
	    else
	        {
		productNameId = hgNextId();
		fprintf(productTab, "%u\t%s\n", productNameId, productName);
		hashAdd(productHash, productName, intToPt(productNameId));
		}
	    fprintf(knownInfoTab, "%s\t%s\t%u\t%u\t%s\t%s\t%s\n",
	        transId, geneId, geneNameId, productNameId, proteinId, ngi, pgi);	
	    }
	}
    }

fclose(productTab);
fclose(geneTab);
fclose(knownInfoTab);
hgLoadTabFile(conn, "productName");
hgLoadTabFile(conn, "geneName");
hgLoadTabFile(conn, "knownInfo");
hgEndUpdate(&conn, "Loaded known gene info from %s", fileName);
}

void hgKnownGene(char *database, char *fileName)
/* hgKnownGene - Load auxiliarry tables on Genie known genes. */
{
hSetDb(database);
hgKnownGeneInDb(fileName);
}

int main(int argc, char *argv[])
/* Process command line. */
{
if (argc != 3)
    usage();
hgKnownGene(argv[1], argv[2]);
return 0;
}
